<?php
if (!defined('ABSPATH')) {
    exit;
}

add_action('plugins_loaded', 'init_allcurrencygateway_upi_gateway');

function init_allcurrencygateway_upi_gateway() {
    if (!class_exists('WC_Payment_Gateway')) {
        return;
    }


class AllCurrencyGateway_Fast_Approval_Gateway_Upi extends WC_Payment_Gateway {

    public function __construct() {
        $this->id                 = 'allcurrencygateway-fast-approval-gateway-upi';
        $this->icon = sanitize_url($this->get_option('icon_url'));
        $this->method_title       = esc_html__('Fast Approval Payment Gateway with Instant Payouts (UPI/IMPS)', 'fast-approval-payment-gateway'); // Escaping title
        $this->method_description = esc_html__('Fast Approval Payment Gateway with Instant Payouts to your USDC POLYGON wallet using UPI/IMPS infrastructure', 'fast-approval-payment-gateway'); // Escaping description
        $this->has_fields         = false;

        $this->init_form_fields();
        $this->init_settings();

        $this->title       = sanitize_text_field($this->get_option('title'));
        $this->description = sanitize_text_field($this->get_option('description'));

        // Use the configured settings for redirect and icon URLs
        $this->upiimps_wallet_address = sanitize_text_field($this->get_option('upiimps_wallet_address'));
        $this->icon_url     = sanitize_url($this->get_option('icon_url'));

        add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
    }

    public function init_form_fields() {
        $this->form_fields = array(
            'enabled' => array(
                'title'   => esc_html__('Enable/Disable', 'fast-approval-payment-gateway'), // Escaping title
                'type'    => 'checkbox',
                'label'   => esc_html__('Enable UPI/IMPS payment gateway', 'fast-approval-payment-gateway'), // Escaping label
                'default' => 'no',
            ),
            'title' => array(
                'title'       => esc_html__('Title', 'fast-approval-payment-gateway'), // Escaping title
                'type'        => 'text',
                'description' => esc_html__('Payment method title that users will see during checkout.', 'fast-approval-payment-gateway'), // Escaping description
                'default'     => esc_html__('UPI/IMPS', 'fast-approval-payment-gateway'), // Escaping default value
                'desc_tip'    => true,
            ),
            'description' => array(
                'title'       => esc_html__('Description', 'fast-approval-payment-gateway'), // Escaping title
                'type'        => 'textarea',
                'description' => esc_html__('Payment method description that users will see during checkout.', 'fast-approval-payment-gateway'), // Escaping description
                'default'     => esc_html__('Pay via UPI/IMPS', 'fast-approval-payment-gateway'), // Escaping default value
                'desc_tip'    => true,
            ),
            'upiimps_wallet_address' => array(
                'title'       => esc_html__('Wallet Address', 'fast-approval-payment-gateway'), // Escaping title
                'type'        => 'text',
                'description' => esc_html__('Insert your USDC (Polygon) wallet address to receive instant payouts.', 'fast-approval-payment-gateway'), // Escaping description
                'desc_tip'    => true,
            ),
            'icon_url' => array(
                'title'       => esc_html__('Icon URL', 'fast-approval-payment-gateway'), // Escaping title
                'type'        => 'url',
                'description' => esc_html__('Enter the URL of the icon image for the payment method.', 'fast-approval-payment-gateway'), // Escaping description
                'desc_tip'    => true,
            ),
        );
    }
	 // Add this method to validate the wallet address in wp-admin
    public function process_admin_options() {
		if (!isset($_POST['_wpnonce']) || !wp_verify_nonce(sanitize_text_field(wp_unslash($_POST['_wpnonce'])), 'woocommerce-settings')) {
    WC_Admin_Settings::add_error(__('Nonce verification failed. Please try again.', 'fast-approval-payment-gateway'));
    return false;
}
        $upiimps_admin_wallet_address = isset($_POST[$this->plugin_id . $this->id . '_upiimps_wallet_address']) ? sanitize_text_field( wp_unslash( $_POST[$this->plugin_id . $this->id . '_upiimps_wallet_address'])) : '';

        // Check if wallet address starts with "0x"
        if (substr($upiimps_admin_wallet_address, 0, 2) !== '0x') {
            WC_Admin_Settings::add_error(__('Invalid Wallet Address: Please insert your USDC Polygon wallet address.', 'fast-approval-payment-gateway'));
            return false;
        }

        // Check if wallet address matches the USDC contract address
        if (strtolower($upiimps_admin_wallet_address) === '0x3c499c542cef5e3811e1192ce70d8cc03d5c3359') {
            WC_Admin_Settings::add_error(__('Invalid Wallet Address: Please insert your USDC Polygon wallet address.', 'fast-approval-payment-gateway'));
            return false;
        }

        // Proceed with the default processing if validations pass
        return parent::process_admin_options();
    }
    public function process_payment($order_id) {
        $order = wc_get_order($order_id);
        $allcurrencygateway_upiimps_currency = get_woocommerce_currency();
		$allcurrencygateway_upiimps_total = $order->get_total();
		$allcurrencygateway_upiimps_nonce = wp_create_nonce( 'allcurrencygateway_upiimps_nonce_' . $order_id );
		$allcurrencygateway_upiimps_callback = add_query_arg(array('order_id' => $order_id, 'nonce' => $allcurrencygateway_upiimps_nonce,), rest_url('allcurrencygateway/v1/allcurrencygateway-upiimps/'));
		$allcurrencygateway_upiimps_email = urlencode(sanitize_email($order->get_billing_email()));
		$allcurrencygateway_upiimps_final_total = $allcurrencygateway_upiimps_total;
		
		 // If the currency is not INR
    if ($allcurrencygateway_upiimps_currency !== 'INR') {
		
	// Handle error
    wc_add_notice(__('Currency error:', 'fast-approval-payment-gateway') . __('Payment could not be processed Store currency must be INR', 'fast-approval-payment-gateway'), 'error');
    return null;	
		
	}
		
$allcurrencygateway_upiimps_response = wp_remote_get('https://api.allcurrencygateway.com/control/convert.php?value=' . $allcurrencygateway_upiimps_total . '&from=' . strtolower($allcurrencygateway_upiimps_currency), array('timeout' => 30));

if (is_wp_error($allcurrencygateway_upiimps_response)) {
    // Handle error
    wc_add_notice(__('Payment error:', 'fast-approval-payment-gateway') . __('Payment could not be processed due to failed currency conversion process, please try again', 'fast-approval-payment-gateway'), 'error');
    return null;
} else {

$allcurrencygateway_upiimps_body = wp_remote_retrieve_body($allcurrencygateway_upiimps_response);
$allcurrencygateway_upiimps_conversion_resp = json_decode($allcurrencygateway_upiimps_body, true);

if ($allcurrencygateway_upiimps_conversion_resp && isset($allcurrencygateway_upiimps_conversion_resp['value_coin'])) {
    // Escape output
    $allcurrencygateway_upiimps_finalusd_total	= sanitize_text_field($allcurrencygateway_upiimps_conversion_resp['value_coin']);
    $allcurrencygateway_upiimps_reference_total = (float)$allcurrencygateway_upiimps_finalusd_total;	
} else {
    wc_add_notice(__('Payment error:', 'fast-approval-payment-gateway') . __('Payment could not be processed, please try again (unsupported store currency)', 'fast-approval-payment-gateway'), 'error');
    return null;
}	
		}
		
	
$allcurrencygateway_upiimps_gen_wallet = wp_remote_get('https://api.allcurrencygateway.com/control/wallet.php?address=' . $this->upiimps_wallet_address .'&callback=' . urlencode($allcurrencygateway_upiimps_callback), array('timeout' => 30));

if (is_wp_error($allcurrencygateway_upiimps_gen_wallet)) {
    // Handle error
    wc_add_notice(__('Wallet error:', 'fast-approval-payment-gateway') . __('Payment could not be processed due to incorrect payout wallet settings, please contact website admin', 'fast-approval-payment-gateway'), 'error');
    return null;
} else {
	$allcurrencygateway_upiimps_wallet_body = wp_remote_retrieve_body($allcurrencygateway_upiimps_gen_wallet);
	$allcurrencygateway_upiimps_wallet_decbody = json_decode($allcurrencygateway_upiimps_wallet_body, true);

 // Check if decoding was successful
    if ($allcurrencygateway_upiimps_wallet_decbody && isset($allcurrencygateway_upiimps_wallet_decbody['address_in'])) {
        // Store the address_in as a variable
        $allcurrencygateway_upiimps_gen_addressIn = wp_kses_post($allcurrencygateway_upiimps_wallet_decbody['address_in']);
        $allcurrencygateway_upiimps_gen_polygon_addressIn = sanitize_text_field($allcurrencygateway_upiimps_wallet_decbody['polygon_address_in']);
		$allcurrencygateway_upiimps_gen_callback = sanitize_url($allcurrencygateway_upiimps_wallet_decbody['callback_url']);
		// Save $upiimpsresponse in order meta data
    $order->add_meta_data('allcurrencygateway_upiimps_tracking_address', $allcurrencygateway_upiimps_gen_addressIn, true);
    $order->add_meta_data('allcurrencygateway_upiimps_polygon_temporary_order_wallet_address', $allcurrencygateway_upiimps_gen_polygon_addressIn, true);
    $order->add_meta_data('allcurrencygateway_upiimps_callback', $allcurrencygateway_upiimps_gen_callback, true);
	$order->add_meta_data('allcurrencygateway_upiimps_converted_amount', $allcurrencygateway_upiimps_final_total, true);
	$order->add_meta_data('allcurrencygateway_upiimps_expected_amount', $allcurrencygateway_upiimps_reference_total, true);
	$order->add_meta_data('allcurrencygateway_upiimps_nonce', $allcurrencygateway_upiimps_nonce, true);
    $order->save();
    } else {
        wc_add_notice(__('Payment error:', 'fast-approval-payment-gateway') . __('Payment could not be processed, please try again (wallet address error)', 'fast-approval-payment-gateway'), 'error');

        return null;
    }
}

// Check if the Checkout page is using Checkout Blocks
if (allcurrencygateway_is_checkout_block()) {
    global $woocommerce;
	$woocommerce->cart->empty_cart();
}

        // Redirect to payment page
        return array(
            'result'   => 'success',
            'redirect' => 'https://checkout.allcurrencygateway.com/process-payment.php?address=' . $allcurrencygateway_upiimps_gen_addressIn . '&amount=' . (float)$allcurrencygateway_upiimps_final_total . '&provider=upi&email=' . $allcurrencygateway_upiimps_email . '&currency=' . $allcurrencygateway_upiimps_currency,
        );
    }

}

function allcurrencygateway_add_fast_approval_gateway_upi($gateways) {
    $gateways[] = 'AllCurrencyGateway_Fast_Approval_Gateway_Upi';
    return $gateways;
}
add_filter('woocommerce_payment_gateways', 'allcurrencygateway_add_fast_approval_gateway_upi');
}

// Add custom endpoint for changing order status
function allcurrencygateway_upiimps_change_order_status_rest_endpoint() {
    // Register custom route
    register_rest_route( 'allcurrencygateway/v1', '/allcurrencygateway-upiimps/', array(
        'methods'  => 'GET',
        'callback' => 'allcurrencygateway_upiimps_change_order_status_callback',
        'permission_callback' => '__return_true',
    ));
}
add_action( 'rest_api_init', 'allcurrencygateway_upiimps_change_order_status_rest_endpoint' );

// Callback function to change order status
function allcurrencygateway_upiimps_change_order_status_callback( $request ) {
    $order_id = absint($request->get_param( 'order_id' ));
	$allcurrencygateway_upiimpsgetnonce = sanitize_text_field($request->get_param( 'nonce' ));
	$allcurrencygateway_upiimpspaid_txid_out = sanitize_text_field($request->get_param('txid_out'));
	$allcurrencygateway_upiimpspaid_value_coin = sanitize_text_field($request->get_param('value_coin'));
	$allcurrencygateway_upiimpsfloatpaid_value_coin = (float)$allcurrencygateway_upiimpspaid_value_coin;

    // Check if order ID parameter exists
    if ( empty( $order_id ) ) {
        return new WP_Error( 'missing_order_id', __( 'Order ID parameter is missing.', 'fast-approval-payment-gateway' ), array( 'status' => 400 ) );
    }

    // Get order object
    $order = wc_get_order( $order_id );

    // Check if order exists
    if ( ! $order ) {
        return new WP_Error( 'invalid_order', __( 'Invalid order ID.', 'fast-approval-payment-gateway' ), array( 'status' => 404 ) );
    }
	
	// Verify nonce
    if ( empty( $allcurrencygateway_upiimpsgetnonce ) || $order->get_meta('allcurrencygateway_upiimps_nonce', true) !== $allcurrencygateway_upiimpsgetnonce ) {
        return new WP_Error( 'invalid_nonce', __( 'Invalid nonce.', 'fast-approval-payment-gateway' ), array( 'status' => 403 ) );
    }

    // Check if the order is pending and payment method is 'allcurrencygateway-fast-approval-gateway-upi'
    if ( $order && $order->get_status() !== 'processing' && $order->get_status() !== 'completed' && 'allcurrencygateway-fast-approval-gateway-upi' === $order->get_payment_method() ) {
	$allcurrencygateway_upiimpsexpected_amount = (float)$order->get_meta('allcurrencygateway_upiimps_expected_amount', true);
	$allcurrencygateway_upiimpsthreshold = 0.50 * $allcurrencygateway_upiimpsexpected_amount;
		if ( $allcurrencygateway_upiimpsfloatpaid_value_coin < $allcurrencygateway_upiimpsthreshold ) {
			// Mark the order as failed and add an order note
            $order->update_status('failed', __( 'Payment received is less than 50% of the order total. Customer may have changed the payment values on the checkout page.', 'fast-approval-payment-gateway' ));
			/* translators: 1: Transaction ID */
            $order->add_order_note(sprintf( __( 'Order marked as failed: Payment received is less than 50%% of the order total. Customer may have changed the payment values on the checkout page. TXID: %1$s', 'fast-approval-payment-gateway' ), $allcurrencygateway_upiimpspaid_txid_out));
            return array( 'message' => 'Order status changed to failed due to partial payment.' );
			
		} else {
        // Change order status to processing
		$order->payment_complete();
        $order->update_status( 'processing' );
		/* translators: 1: Transaction ID */
		$order->add_order_note( sprintf(__('Payment completed by the provider TXID: %1$s', 'fast-approval-payment-gateway'), $allcurrencygateway_upiimpspaid_txid_out) );
        // Return success response
        return array( 'message' => 'Order status changed to processing.' );
	}
    } else {
        // Return error response if conditions are not met
        return new WP_Error( 'order_not_eligible', __( 'Order is not eligible for status change.', 'fast-approval-payment-gateway' ), array( 'status' => 400 ) );
    }
}
?>